@@ -1,4 +1,4 @@
-/* $Id: checks.c 2693 2010-01-22 09:17:08Z rb $ */
+/* $Id: checks.c 4029 2010-09-29 11:57:19Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -257,6 +257,8 @@ void runInfoCheck(unsigned int counter) {
 
     /* No init */
 
+    rv = C_GetInfo(NULL_PTR);
+    assert(rv == CKR_CRYPTOKI_NOT_INITIALIZED);
     rv = C_GetSlotList(CK_FALSE, NULL_PTR, NULL_PTR);
     assert(rv == CKR_CRYPTOKI_NOT_INITIALIZED);
     rv = C_GetSlotInfo(slotInvalid, NULL_PTR);
@@ -268,14 +270,6 @@ void runInfoCheck(unsigned int counter) {
     rv = C_GetMechanismInfo(slotInvalid, CKM_VENDOR_DEFINED, NULL_PTR);
     assert(rv == CKR_CRYPTOKI_NOT_INITIALIZED);
 
-    /* C_GetInfo */
-
-    rv = C_GetInfo(NULL_PTR);
-    assert(rv == CKR_ARGUMENTS_BAD);
-
-    rv = C_GetInfo(&ckInfo);
-    assert(rv == CKR_OK);
-
     /* C_GetFunctionList */
     
     rv = C_GetFunctionList(NULL_PTR);
@@ -284,11 +278,19 @@ void runInfoCheck(unsigned int counter) {
     rv = C_GetFunctionList(&ckFuncList);
     assert(rv == CKR_OK);
 
-    /* C_GetSlotList */
+    /* C_GetInfo */
 
     rv = C_Initialize(NULL_PTR);
     assert(rv == CKR_OK);
 
+    rv = C_GetInfo(NULL_PTR);
+    assert(rv == CKR_ARGUMENTS_BAD);
+
+    rv = C_GetInfo(&ckInfo);
+    assert(rv == CKR_OK);
+
+    /* C_GetSlotList */
+
     rv = C_GetSlotList(CK_FALSE, NULL_PTR, NULL_PTR);
     assert(rv == CKR_ARGUMENTS_BAD);
     rv = C_GetSlotList(CK_FALSE, NULL_PTR, &ulSlotCount);
@@ -527,7 +529,7 @@ void runUserCheck(unsigned int counter) {
     rv = C_Login(hSession[0], CKU_CONTEXT_SPECIFIC, userPIN, sizeof(userPIN) - 1);
     assert(rv == CKR_OK);
     rv = C_Login(hSession[1], CKU_SO, soPIN, sizeof(soPIN) - 2);
-    assert(rv == CKR_USER_TOO_MANY_TYPES);
+    assert(rv == CKR_USER_ANOTHER_ALREADY_LOGGED_IN);
     rv = C_Logout(hSession[0]);
     assert(rv == CKR_OK);
     rv = C_Login(hSession[1], CKU_SO, soPIN, sizeof(soPIN) - 2);
@@ -541,7 +543,7 @@ void runUserCheck(unsigned int counter) {
     rv = C_Login(hSession[1], CKU_CONTEXT_SPECIFIC, soPIN, sizeof(soPIN) - 1);
     assert(rv == CKR_OK);
     rv = C_Login(hSession[1], CKU_USER, userPIN, sizeof(userPIN) - 2);
-    assert(rv == CKR_USER_TOO_MANY_TYPES);
+    assert(rv == CKR_USER_ANOTHER_ALREADY_LOGGED_IN);
 
     /* C_Logout */
 
