@@ -1,4 +1,4 @@
-/* $Id: SoftHSMInternal.cpp 2539 2009-12-01 10:47:49Z rb $ */
+/* $Id: SoftHSMInternal.cpp 4029 2010-09-29 11:57:19Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -45,6 +45,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 // Includes for the crypto library
 #include <botan/pipe.h>
@@ -70,6 +72,13 @@ SoftHSMInternal::SoftHSMInternal(bool threading, CK_CREATEMUTEX cMutex,
   this->createMutex(&pHSMMutex);
 
   slots = new SoftSlot();
+
+  time_t rawtime;
+  time(&rawtime);
+  char dateTime[15];
+  strftime(dateTime, 15, "%Y%m%d%H%M%S", gmtime(&rawtime));
+
+  snprintf(appID, 32, "%s-%010i", dateTime, getpid());
 }
 
 SoftHSMInternal::~SoftHSMInternal() {
@@ -116,7 +125,7 @@ CK_RV SoftHSMInternal::openSession(CK_SLOT_ID slotID, CK_FLAGS flags, CK_VOID_PT
 
   for(int i = 0; i < MAX_SESSION_COUNT; i++) {
     if(sessions[i] == NULL_PTR) {
-      sessions[i] = new SoftSession(flags & CKF_RW_SESSION, currentSlot);
+      sessions[i] = new SoftSession(flags & CKF_RW_SESSION, currentSlot, appID);
 
       // Check that we have a connection
       if(sessions[i]->db == NULL_PTR) {
@@ -252,7 +261,7 @@ CK_RV SoftHSMInternal::login(CK_SESSION_HANDLE hSession, CK_USER_TYPE userType,
     case CKU_SO:
       // Only one user type can be logged in
       CHECK_DEBUG_RETURN(session->currentSlot->userPIN != NULL_PTR, "C_Login", "A normal user is already logged in",
-                         CKR_USER_TOO_MANY_TYPES);
+                         CKR_USER_ANOTHER_ALREADY_LOGGED_IN);
 
       // Check that we have no R/O session with the slot
       for (int i = 0; i < MAX_SESSION_COUNT; i++) {
@@ -266,7 +275,7 @@ CK_RV SoftHSMInternal::login(CK_SESSION_HANDLE hSession, CK_USER_TYPE userType,
     case CKU_USER:
       // Only one user type can be logged in
       CHECK_DEBUG_RETURN(session->currentSlot->soPIN != NULL_PTR, "C_Login", "A SO is already logged in",
-                         CKR_USER_TOO_MANY_TYPES);
+                         CKR_USER_ANOTHER_ALREADY_LOGGED_IN);
 
       CHECK_DEBUG_RETURN(session->currentSlot->hashedUserPIN == NULL_PTR, "C_Login", "The normal user PIN is not initialized",
                          CKR_USER_PIN_NOT_INITIALIZED);
